%% SIL Simulation Monte Carlo Setup
%  The commands in this script will set conditions for the SIL simulation
%  for a range of environmental conditions and model uncertainties. Note
%  that this script doesn't run the sim, it only sets the conditions,
%  assuming you've already run setup.
%
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
%
% SVN Info: $Id: SIL_montecarlo.m 314 2011-04-05 16:53:30Z murch $

%% Vary Sensor Noise and Turbulence 
% Noise off, Turbulence off
AC.Sensors.NoiseOn = 0;
Env.Winds.TurbulenceOn = 0;

% RUN SIM HERE

% Noise on, Turbulence off (default)
AC.Sensors.NoiseOn = 1;
Env.Winds.TurbulenceOn = 0;

% RUN SIM HERE

% Noise off, Turbulence on
AC.Sensors.NoiseOn = 0;
Env.Winds.TurbulenceOn = 1;

% RUN SIM HERE

% Noise on, Turbulence on
AC.Sensors.NoiseOn = 1;
Env.Winds.TurbulenceOn = 1;

% RUN SIM HERE

% Return to default, Noise on, Turbulence off
AC.Sensors.NoiseOn = 1;
Env.Winds.TurbulenceOn = 0;

%% Model Uncertainty: Vary Pitch Rate Damping
% Store baseline value
Cmq = AC.Aero.Cm.q;

% Increase 50%
AC.Aero.Cm.q = Cmq*1.5;

% RUN SIM HERE

% Decrease 50%
AC.Aero.Cm.q = Cmq*.5;

% RUN SIM HERE

% Baseline
AC.Aero.Cm.q = Cmq;

%% Model Uncertainty: Vary Roll Damping
% Store baseline value
Clp = AC.Aero.Cl.p;

% Increase 25%
AC.Aero.Cl.p = Clp*1.25;

% RUN SIM HERE

% Decrease 25%
AC.Aero.Cl.p = Clp*.75;

% RUN SIM HERE

% Baseline
AC.Aero.Cl.p = Clp;

%% Model Uncertainty: Vary Yaw Damping
% Store baseline value
Cnr = AC.Aero.Cn.r;

% Increase 25%
AC.Aero.Cn.r = Cnr*1.25;

% RUN SIM HERE

% Decrease 25%
AC.Aero.Cn.r = Cnr*.75;

% RUN SIM HERE

% Baseline
AC.Aero.Cn.r = Cnr;