%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Script to plot flight and simulation data. 
%  
%   David Escobar Sanabria
%   Aerospace Engineering and Mechanics 
%   University of Minnesota, Twin Cities
%   descobar@aem.umn.edu
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = plotMatching(fltpackt, simData,initSample, finalSample, Ts, dataType) 
 
D2R = pi/180; % degrees to radians
R2D = 180/pi; % Radians to deggrees

% Figures counter (Algorithm)
global fig;
fig=1;

%% Definition of flight test signal offset 
% This is the offset of the flight data respect to the simulated data.
offset =5 ; %R2D*mode(fltpackt.pitch_ref)


%% Match and plot the flight test and simulation data

%% Plots for longitudinal dynamics:



%vectorSignalTest= [R2D*fltpackt.de(initSample:finalSample)
%R2D*fltpackt.q(initSample:finalSample) fltpackt.Pt(initSample:finalSample)
%fltpackt.Ps(initSample:finalSample) ];

% Simulation data related to Lateral dynamics
vectorSignalSim = [simData.elevator-10 simData.q simData.V_s simData.alt];
% Flight data related to Lateral dynamics
vectorSignalTest= [R2D*fltpackt.de(initSample:finalSample) R2D*fltpackt.q(initSample:finalSample) fltpackt.ias(initSample:finalSample) fltpackt.alt(initSample:finalSample) ];


% Vector of names: Make sure the dimension of the string is 18
vectorNames = [ 'Elevator [deg]    '; 'Pitch rate [deg/s]'; 'Air speed [m/s]   '; 'Altitude [m]      ' ];

offset_theta_cmd = mode(simData.theta_cmd);
offset_theta = mode(simData.theta);
% Organization, matching and ploting of data
compDoublePlot( simData.theta-offset_theta, simData.theta_cmd -offset_theta_cmd, simData.time, vectorSignalSim,R2D*fltpackt.the(initSample:finalSample),R2D*fltpackt.pitch_ref(initSample:finalSample) ,fltpackt.time(initSample:finalSample), vectorSignalTest, offset, '\theta',vectorNames, Ts);


%% Plots for lateral dynamics:
offset =R2D*mode(fltpackt.roll_ref);

% Simulation data related to Lateral dynamics
vectorSignalSim = [-simData.aileron -simData.rudder simData.p simData.r];

% Flight data related to Lateral dynamics

vectorSignalTest= [R2D*fltpackt.da_r(initSample:finalSample) R2D*fltpackt.dr(initSample:finalSample) R2D*fltpackt.p(initSample:finalSample) R2D*fltpackt.r(initSample:finalSample) ];

% Vector of names: Make sure the dimension of the string is 18
vectorNames = [ 'Aileron [deg]     '; 'Rudder [deg]      '; 'Roll rate [deg/s] '; 'Yaw rate [deg/s]  '];

% Organization, matching and ploting of data
compDoublePlot( simData.phi, simData.phi_cmd, simData.time,vectorSignalSim,R2D*fltpackt.phi(initSample:finalSample),R2D*fltpackt.roll_ref(initSample:finalSample) ,fltpackt.time(initSample:finalSample), vectorSignalTest, offset, '\phi',vectorNames, Ts);


end