/*
 * empty_control.c
 *
 *  Created on: Jan 1, 2011
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: empty_control.c 253 2011-01-20 22:01:35Z murch $
 */

#include "control_law_interface.h"
#include "control_defs.h"
#include <math.h> 

/// Definition of variables: ****************************************************
static double Ts = 0; // Sample time
/// ****************************************************************

// Angles, rates and accelarations:  ********************************
static double phi = 0; // Roll angle
static double theta = 0; // Pitch angle
static double psi = 0; // Yaw angle
static double p = 0; // Roll rate
static double q = 0; // Pitch rate
static double r = 0; // Yaw rate
static double ax = 0 ; //accel. x direction
static double ay = 0 ; //accel. y direction
static double az = 0 ; //accel. z direction
static double h = 0;
static double ias = 0;
static double lat = 0;
static double lon = 0;
static double alt = 0;
static double vn = 0;
static double ve = 0;
static double vd = 0;
/// ****************************************************************

/// Control outputs:  **********************************************
// : aileron, elevator, rudder.
static double commands[4];
static double delta_elev=0, delta_ail=0, delta_rudd=0, delta_thr =0;
//  ****************************************************************

/// References: ****************************************************
static double theta_reference;
static double phi_reference;
static double yaw_rate;
// *****************************************************************

/// Return control outputs based on references and feedback signals.
extern double *get_control_outputs(double *feedback, double *reference, double time, double sample_time) {

	Ts    = sample_time;
	phi   = feedback[0];
	theta = feedback[1];
	psi   = feedback[2];
	p     = feedback[3]; // Roll rate
	q     = feedback[4]; // Pitch rate
	r     = feedback[5]; // Yaw rate
	ax    = feedback[6]; // Accel. x
	ay    = feedback[7]; // Accel. y
    az    = feedback[8]; // Accel. z
	h     = feedback[9];
	ias   = feedback[10];
	lat   = feedback[11];
	lon   = feedback[12];
	alt   = feedback[13];
	vn    = feedback[14];
	ve    = feedback[15];
	vd    = feedback[16];

    phi_reference = reference[0];
    theta_reference = reference[1];
    yaw_rate = reference[2];

    // Control law: ***********************************************************
    // Here: Compute delta_ail, delta_rudd , delta_elev and delta_thr

      /// Control implementation ...

    // ************************************************************************

    commands[0] =  delta_ail;  // Aileron
    commands[1] =  delta_elev; // Elevator
    commands[2] =  delta_rudd; // Rudder
    commands[3] =  delta_thr;  // Throttle

	return commands;
}

// Reset of controller
extern void reset_control(){
	// Here: code to reset the controller

}
