/*
 * FILE: sensors.h
 * DESCRIPTION:
 *
 *
 *  Created on: Feb 27, 2010
 *      Author: Rohit Pandita, Zhefeng Li
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: sensors.h 253 2011-01-20 22:01:35Z murch $
 */

#ifndef SENSORS_H_
#define SENSORS_H_

#include "AirData/HoneywellPitot.c"
#include "AirData/HoneywellAltimeter.c"

// SPI port for IMU
#define SPIPSC_PORT1 	"/dev/spipsc1"


#define ABS_PRESS_MAX		30
#define DIFF_PRESS_MAX		1
#define psi2kpa 			6.894757;

cyg_io_handle_t hSPI;
//#define verbose		// enable spi, i2c debug messages

/* function prototypes */
int init_spi();
int read_spi(struct imu *imupkt);
void init_i2c();
void read_pspd (struct imu *imupkt);
double convert_ad( cyg_uint16 raw, double factor );

extern cyg_i2c_bus mpc5200_i2c_bus1;
extern cyg_i2c_bus mpc5200_i2c_bus0;

cyg_i2c_device i2c_eeprom = {            
      .i2c_bus        = &mpc5200_i2c_bus1,
      .i2c_address    = 0xA4,            
      .i2c_flags      = 0x00,            
      .i2c_delay      = CYG_I2C_DEFAULT_DELAY
  };

/* USR40X ultrasonic ranging sensor */
cyg_i2c_device i2c_usonic = {            
      .i2c_bus        = &mpc5200_i2c_bus1,
      .i2c_address    = 0xA6,            
      .i2c_flags      = 0x00,            
      .i2c_delay      = CYG_I2C_DEFAULT_DELAY
  };
  

/* RTC8564 realtime clock */
cyg_i2c_device i2c_rtc = {            
      .i2c_bus        = &mpc5200_i2c_bus1,
      .i2c_address    = 0xA2,            
      .i2c_flags      = 0x00,            
      .i2c_delay      = CYG_I2C_DEFAULT_DELAY
  };
  
cyg_i2c_device i2c_honeywell_static = {            
      .i2c_bus        = &mpc5200_i2c_bus0,
      .i2c_address    = 0xF0,            
      .i2c_flags      = 0x00,            
      .i2c_delay      = CYG_I2C_DEFAULT_DELAY
  };
  
cyg_i2c_device i2c_honeywell_pitot = {            
      .i2c_bus        = &mpc5200_i2c_bus0,
      .i2c_address    = 0x50,            // for 0x28
      .i2c_flags      = 0x00,            
      .i2c_delay      = CYG_I2C_DEFAULT_DELAY
  };

// pressure sensors
HoneywellAltimeter myAltimeter;
HoneywellPitot myPitot;

#endif /* SENSORS_H_ */
